<?php

namespace Drupal\tideways\StackMiddleware;

use Symfony\Component\HttpKernel\Event\FilterControllerEvent;
use Symfony\Component\HttpKernel\Event\FilterResponseEvent;
use Symfony\Component\HttpKernel\Event\PostResponseEvent;
use Symfony\Component\HttpKernel\Event\GetResponseForExceptionEvent;

use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Exception\HttpExceptionInterface;
use Symfony\Component\HttpKernel\KernelEvents;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\HttpKernelInterface;

/**
 * Provides a HTTP middleware.
 */
class Tideways implements EventSubscriberInterface, HttpKernelInterface  {

  /**
   * The wrapped HTTP kernel.
   *
   * @var \Symfony\Component\HttpKernel\HttpKernelInterface
   */
  protected $httpKernel;

  /**
   * If the Tideways extension is enabled.
   *
   * @var boolean
   */
  protected $enabled;

  /**
   * Constructs a Tideways object.
   *
   * @param \Symfony\Component\HttpKernel\HttpKernelInterface $kernel
   *   The decorated kernel.
   * @param mixed $optional_argument
   *   (optional) An optional argument.
   */
  public function __construct(HttpKernelInterface $http_kernel) {
    $this->enabled = class_exists(\Tideways\Profiler::class);
    $this->httpKernel = $http_kernel;
  }

  /**
   * {@inheritdoc}
   */
  public function handle(Request $request, $type = self::MASTER_REQUEST, $catch = TRUE) {
    if ($this->enabled) {
      // We do this early because not all requests end up at a controller...
      \Tideways\Profiler::setTransactionName($request->getPathInfo());
    }
    return $this->httpKernel->handle($request, $type, $catch);
  }

  /**
   * Summary of onKernelController
   *
   * @param FilterControllerEvent $event
   * @return void
   */
  public function onKernelController(FilterControllerEvent $event) {
    if (!$this->enabled) {
      return;
    }
    $request = \Drupal::requestStack()->getCurrentRequest();
    $atts = $request->attributes;
    /** @var \Symfony\Component\Routing\Route */
    $route = $atts->get('_route_object');
    // Update the transaction name to something more meaningful.
    \Tideways\Profiler::setTransactionName($atts->get('_controller'));
  }

  /**
   * Summary of onKernelException
   *
   * @param GetResponseForExceptionEvent $event
   * @return void
   */
  public function onKernelException(GetResponseForExceptionEvent $event) {
    if (!$this->enabled) {
      return;
    }
    \Tideways\Profiler::logException($event->getException());
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    if (!class_exists(\Tideways\Profiler::class)) {
      // If tideways extension not enabled, do not
      // subscribe to events.
      return [];
    }
    $events[KernelEvents::CONTROLLER][] = ['onKernelController', -999];
    $events[KernelEvents::EXCEPTION][] = ['onKernelException', -999];
    return $events;
  }
}